/*
 * libzdt - zdt's own C Library.
 * Copyright (C) 1999,2000 Jeremy Tregunna. All rights reserved.
 *
 * libzdt is distributed under a modified version of the BSD license.
 * You should have received a copy of this license in the Documentation
 * directory.
 */
#include <zdt/Types.h>
#include <zdt/ZString.h>
/* #include <zdt/ZObject.h> */
#include <objc/objc.h>

/* Standard headers */
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/types.h>
#include <ctype.h>

@implementation ZString

- (const char *)cString
{
	return(c_string);
}

- (unsigned int)length
{
	return(len);
}

/* - (int)ZPrintString:(NXConstantString *)format,... */
- (int)ZPrintString:(NXConstantString *)format,...
{
	int done;
	va_list va;

	va_start(va,(const char *)format);

	done = vfprintf(stdout,(const char *)format,va);
	va_end(va);

	return(done);
}

- (int)ZPrintError:(NXConstantString *)format,...
{
	int done;
	va_list va;

	va_start(va,(const char *)format);

	done = vfprintf(stderr,(const char *)format,va);
	va_end(va);

	return(done);
}

/*
 * Determine if littles is contained in bigs (ignoring case).
 * If so, return pointer to the littles in bigs.
 * If not, return NULL.
 */
- (char *)stristr:(char *)bigs littles:(char *)littles
{
	char *st = bigs;
	char *p,*q;

	while(1) {
		if (!*st) {
			return NULL;
		}
		p = littles;
		q = st;

		while((*p) && (*q) && (upcase(*p) == upcase(*q))) {
			p++;
			q++;
		}

		if ((!*q) && (*p)) {
			return NULL;	/* Premature end of bigs */
		}
		if (!*p) {
			return st;	/* Found it */
		}

		st++;	/* Try again */
	}
}

- (int)strnstr:(char *)buffer string:(char *)aString
{
	char *__tempo;

	__tempo = (char *)malloc(strlen(buffer) + 2); /* nmalloc() */
	memset(__tempo,0,sizeof(__tempo));
	strcpy(__tempo,buffer);

	while(strlen(__tempo) != 0) {
		if ((strlen(aString)) >= (strlen(__tempo))) {
			free(__tempo); /* nfree() */
			return 0;
		}
		if (strncasecmp(__tempo,aString,strlen(aString)) == 0) {
			free(__tempo); /* nfree() */
			return 1;
		}
		strcpy(__tempo,&__tempo[1]);
	}
	free(__tempo); /* nfree() */

	return 0;
}

- (uint8 *)p2istr:(uint8 *)aString
{
	int length = *aString;

	memmove(aString,aString + 1,length);
	aString[length] = 0;

	return (uint8 *)aString;
}

- (uint8 *)i2pstr:(uint8 *)aString
{
	int length = strlen((const char *)aString);

	if (length > 255) {
		length = 255;
	}
	memmove(aString + 1,aString,length);
	aString[0] = length;

	return (uint8 *)aString;
}

- (void)p2istrncpy:(uint8 *)cString pString:(const uint8 *)pString maxLength:(int32)maxLength
{
	int length = *pString;

	if (length >= maxLength) {
		length = maxLength - 1;
	}
	memmove(cString,pString + 1,length);
	cString[length] = 0;
}

- (void)i2pstrncpy:(uint8 *)pString cString:(const uint8 *)cString maxLength:(int32)maxLength
{
	int length = strlen((const char *)cString);

	if (length >= maxLength) {
		length = maxLength - 1;
	}
	memmove(pString + 1,cString,length);
	pString[0] = length;
}

- (void)ZMemoryCopy:(uint8 *)dest source:(const uint8 *)source length:(unsigned int)length
{
	if (length == 0) {
		return;
	}
	do {
		*dest++ = *source++;	/** ??? to be unrolled? */
	} while(--length != 0);
}

- (int)ZMemoryCompare:(const uint8 *)s1 s2:(const uint8 *)s2 length:(unsigned int)length
{
	uint j;

	for(j = 0; j < length; j++) {
		if (s1[j] != s2[j]) {
			return 2*(s1[j] > s2[j])-1;
		}
	}

	return 0;
}

- (void)ZMemoryZero:(uint8 *)dest length:(unsigned int)length
{
	if (length == 0) {
		return;
	}
	do {
		*dest++ = 0;
	} while(--length != 0);
}

- (voidpf)ZCAlloc:(voidpf)opaque items:(unsigned)items size:(unsigned)size
{
	if (opaque) {
		items += size - size; /* make compiler happy */
	}
	return (voidpf)calloc(items,size);
}

- (void)ZCFree:(voidpf)opaque pointer:(voidpf)pointer
{
	free(pointer);
	if (opaque) {
		return;
	}
}

- (void *)ZSearchMemory:(const void *)mem c:(int)c length:(int)length
{
	const char *pos = ((const char *)mem) + length;

	while((--pos) >= (const char *)mem) {
		if ((*pos) == (char)c) {
			return (void *)pos;
		}
	}

	return 0;
}

- (int)ZCaseInsensitiveMemoryCompare:(const char *)aString anotherString:(const char *)anotherString length:(size_t)length
{
	if (aString == anotherString) {
		return 0;
	}
	if (!aString) {
		return -1;
	}
	if (!anotherString) {
		return -1;
	}

	while(length > 0) {
		if (tolower(*aString) < tolower(*anotherString)) {
			return -1;
		}
		if (tolower(*aString) > tolower(*anotherString)) {
			return 1;
		}

		length--;
		aString++;
		anotherString++;
	}

	return 0;
}

- (int)DoNumericCompare:(const char **)aString anotherString:(const char **)anotherString length:(size_t *)length
{
	int res = 0;
	BOOL aNumber = TRUE;
	BOOL anotherNumber = TRUE;

	do {
		if (!res) {
			if (**aString < **anotherString) {
				res = -1;
			}
			if (**aString > **anotherString) {
				res = 1;
			}
		}

		if (length) {
			(*length)--;
			if (*length <= 0) {
				return res;
			}
		} else {
			if (**aString == 0 && **anotherString == 0) {
				return res;
			}
		}

		(*aString)++;
		(*anotherString)++;

		aNumber = !(**aString < '0' || **aString > '9');
		anotherNumber = !(**anotherString < '0' || **anotherString > '9');

		if (aNumber != anotherNumber) {
			if (aNumber) {
				return 1;
			}
			if (anotherNumber) {
				return -1;
			}
		}
	} while(aNumber && anotherNumber);

	return res;
}

- (int)DoStringCompare:(const char **)aString anotherString:(const char **)anotherString length:(size_t *)length
{
	do {
		if (**aString < **anotherString) {
			return -1;
		}
		if (**aString > **anotherString) {
			return 1;
		}

		if (length) {
			(*length)--;
			if (*length <= 0) {
				return 0;
			}
		} else {
			if (**aString == 0 && **anotherString == 0) {
				return 0;
			}
		}

		(*aString)++;
		(*anotherString)++;
	} while(**aString < '0' || **aString > '9' || **anotherString < '0' || **anotherString > '9');

	return 0;
}

- (int)DoCaseInsensitiveStringCompare:(const char **)aString anotherString:(const char **)anotherString length:(size_t *)length
{
	do {
		if (tolower(**aString) < tolower(**anotherString)) {
			return -1;
		}
		if (tolower(**aString) > tolower(**anotherString)) {
			return 1;
		}

		if (length) {
			(*length)--;
			if (*length <= 0) {
				return 0;
			}
		} else {
			if (**aString == 0 && **anotherString == 0) {
				return 0;
			}
		}

		(*aString)++;
		(*anotherString)++;
	} while(**aString < '0' || **aString > '9' || **anotherString < '0' || **anotherString > '9');

	return 0;
}

- (int)NumericStringCompare:(const char *)aString anotherString:(const char *)anotherString
{
	BOOL alpha = *aString < '0' || *aString > '9' || *anotherString < '0' || *anotherString > '9';
	int res = 0;

	if (aString == anotherString) {
		return 0;
	}
	if (!aString) {
		return -1;
	}
	if (!anotherString) {
		return 1;
	}

	do {
		/* Perform appropriate type of string comparasin. */
		if (alpha) {
			res = [self DoStringCompare:&aString anotherString:&anotherString length:0];
		} else {
			res = [self DoNumericCompare:&aString anotherString:&anotherString length:0];
		}

		/* A return meant either the end of the strings, 'res' is
		   non-zero, or we need to switch the type of comparasin. */
		alpha = !alpha;
	} while(res == 0 && *aString && *anotherString);

	return res;
}

- (int)CaseInsensitiveNumericStringCompare:(const char *)aString anotherString:(const char *)anotherString
{
	BOOL alpha = *aString < '0' || *aString > '9' || *anotherString < '0' || *anotherString > '9';
	int res = 0;

	if (aString == anotherString) {
		return 0;
	}
	if (!aString) {
		return -1;
	}
	if (!anotherString) {
		return 1;
	}

	do {
		if (alpha) {
			res = [self DoCaseInsensitiveStringCompare:&aString anotherString:&anotherString length:0];
		} else {
			res = [self DoNumericCompare:&aString anotherString:&anotherString length:0];
		}

		alpha = !alpha;
	} while(res == 0 && *aString && *anotherString);

	return res;
}

- (int)NumericMemoryCompare:(const char *)aString anotherString:(const char *)anotherString length:(size_t)length
{
	BOOL alpha = *aString < '0' || *aString > '9' || *anotherString < '0' || *anotherString > '9';
	int res = 0;

	if (aString == anotherString) {
		return 0;
	}
	if (!aString) {
		return -1;
	}
	if (!anotherString) {
		return 1;
	}
	if (length <= 0) {
		return 0;
	}

	do {
		if (alpha) {
			res = [self DoStringCompare:&aString anotherString:&anotherString length:&length];
		} else {
			res = [self DoNumericCompare:&aString anotherString:&anotherString length:&length];
		}

		alpha = !alpha;
	} while(res == 0 && length > 0);

	return res;
}

- (int)CaseInsensitiveNumericMemoryCompare:(const char *)aString anotherString:(const char *)anotherString length:(size_t)length
{
	BOOL alpha = *aString < '0' || *aString > '9' || *anotherString < '0' || *anotherString > '9';
	int res = 0;

	if (aString == anotherString) {
		return 0;
	}
	if (!aString) {
		return -1;
	}
	if (!anotherString) {
		return 1;
	}
	if (length <= 0) {
		return 0;
	}

	do {
		if (alpha) {
			[self DoCaseInsensitiveStringCompare:&aString anotherString:&anotherString length:&length];
		} else {
			[self DoNumericCompare:&aString anotherString:&anotherString length:&length];
		}

		alpha = !alpha;
	} while(res == 0 && length> 0);

	return res;
}

- (void)rmspace:(char *)aString
{
	char *p;

	/* Wipe end of string */
	for(p = aString + strlen(aString) - 1; ((whitespace(*p)) && (p >= aString)); p--);

	if (p != aString + strlen(aString) - 1) {
		*(p + 1) = 0;
	}

	for(p = aString; ((whitespace(*p)) && (*p)); p++);
	if (p != aString) {
		strcpy(aString,p);
	}
}

/* Split character */
- (void)SplitCharacter:(char *)first rest:(char *)rest divider:(char)divider
{
	char *p;

	p = strchr(rest,divider);
	if (p == NULL) {
		if ((first != rest) && (first != NULL)) {
			first[0] = 0;
		}
		return;
	}

	*p = 0;
	if (first != NULL) {
		strcpy(first,rest);
	}
	if (first != rest) {
		strcpy(rest,p + 1);
	}
}

- (void)Split:(char *)first rest:(char *)rest
{
	[self SplitCharacter:first rest:rest divider:' '];
}

- (void)NSplit:(char *)first rest:(char *)rest
{
	[self Split:first rest:rest];
	if (first != rest) {
		if (!first[0]) {
			strcpy(first,rest);
			rest[0] = 0;
		}
	}
}

/*
 * When a fatal signal is received, call [ZString Fatal::]
 * It will exit whatever it was called from with the specified return code.
 */
- (void)Fatal:(NXConstantString *)aString recoverable:(int)recoverable
{
	if (recoverable == 0) {
		[self ZPrintString:@"%s. Ignoring due to error code = %d\n",aString,recoverable];
	} else {
		[self ZPrintError:@"%s.\nExitting with error code = %d\n",aString,recoverable];
		exit(recoverable);
	}
}

/*** SIGNALS!!! Oh Fun! ***/

- (void)GotSIGBUS:(int)z
{
	[self Fatal:@"BUS Error." recoverable:1];
}

- (void)GotSIGSEGV:(int)z
{
	[self Fatal:@"Received SIGSEGV, Segmentation Violation." recoverable:1];
}

- (void)GotSIGFPE:(int)z
{
	[self Fatal:@"Floating Point Exception." recoverable:1];
}

- (void)GotSIGTERM:(int)z
{
	[self Fatal:@"Received the Terminate signal." recoverable:1];
}

- (void)GotSIGQUIT:(int)z
{
	[self Fatal:@"Received Quit signal." recoverable:1];
}

- (void)GotSIGHUP:(int)z
{
	[self Fatal:@"Got Hangup signal." recoverable:0];
}

- (void)GotSIGALRM:(int)z
{
	[self Fatal:@"Received SIGALRM." recoverable:0];
	return;
}

@end
